pro convert_cd2mr,wvcd,ozcd,cocd,pres,psurf,botlev,wvmr,ozmr,comr
; this routine converts trace gas profiles in column density 
; to mixing ratio
; input:
;   wvcd = water vapor column density [molec/cm2]
;   ozcd = ozone column density [molec/cm2]
;   pres = pressure array on which cd profiles are defined
; output: 
;   wvmr = water vapor mass mixing ratio [g/kg]
;   ozmr = ozone mixing ratio [ppb]
; -------------------

; define constants
  navog=6.02214199e+23; NIST Avogadro's number (molecules/mole)
  nloschmidt=2.6867775e+19 ; NIST Loschmidt's molec/cm^3 per atm @ stdP
  t_std=273.15; standard temperature [K]
  mw_wv=18.0151; gm/mole water
  mw_oz=47.9982; gm/mole ozone
  mw_co=28.0104; gm/mole carbon monoxide
  mw_d=28.9644; gm/mole dry air
  g_std=980.664; acceleration of gravity, cm/s^2
  p_std=1013.25; standard pressure mbar
  cdair=(1000.*p_std*navog)/(mw_d*g_std); =2.14824e+25
  wv_eps=mw_wv/mw_d
  oz_eps=mw_oz/mw_d;
  co_eps=mw_co/mw_d;

; extract NUCAPS vertical pressure grid to local variable
  nlev = n_elements(pres)
; determine size of NUCAPS arrays, i.e., number of retrievals
  s=size(wvcd); [nlev x nobs]
  nobs = s(2)
  print,'size of wvcd array ',nobs
  print,'pressure levels ',nlev

; define conversion arrays
  wvmr=fltarr(nlev,nobs)*!Values.F_NAN
  ozmr=fltarr(nlev,nobs)*!Values.F_NAN
  comr=fltarr(nlev,nobs)*!Values.F_NAN

; calculate cdd() – the layer column density for dry air
; ----------------------------------------------
; cdt(L) is the molecules/cm^2 of air in that layer
; cdd(L) is the molecules/cm^2 of dry air in that layer
; loop over each retrieval and interpolate profile by profile

; define delta pressure layer thinckness array
  deltap=fltarr(nlev)
  deltap(0)=pres(0)
  deltap(1:nlev-1)=pres(1:nlev-1)-pres(0:nlev-2)
; do conversion

  for iobs=0, nobs-1 do begin
   sfc = botlev(iobs)-1; array index where pressure array is at surface
   

   for L=0,sfc-1 do begin
     wvmr(L,iobs)=wvcd(L,iobs) / ((cdair * deltap(L) / p_std) / wv_eps); [molec/cm2] to [g/g]
     ozmr(L,iobs)=ozcd(L,iobs) / ((cdair * deltap(L) / p_std) / oz_eps) ; [molec/cm2] to [g/g]
     comr(L,iobs)=cocd(L,iobs) / ((cdair * deltap(L) / p_std) / co_eps) ; [molec/cm2] to [g/g]
   endfor
   ozmr(*,iobs) = ozmr(*,iobs)*1.0e09; [g/g] to [ppb]
   comr(*,iobs) = comr(*,iobs)*1.0e09; [g/g] to [ppb]
   wvmr(*,iobs) = wvmr(*,iobs) ; [g/g] to [g/kg]

  endfor
end
