pro interpolate_gases,nucaps,stdplev,botlev,stdwv,stdco,stdoz
;
; input is molec/cm2
; output is molec/cm2
;
; ---------------------------------
; procedure to interpolate a nucaps h2o column density profile from 
; 100 levels to the standard NWP levels 
; 
; input: wvcd and ozcd [molec/cm2] from convert_mr2cd procedure
; output: interpolated column density arrays
; ----------------------------------
  fillval=!Values.F_NAN

; extract NUCAPS vertical pressure grid to local variable
  pres = nucaps.pressure(*,0)
  wvcd = nucaps.h2o
  ozcd = nucaps.o3
  cocd = nucaps.co
  nlev = n_elements(pres)

; calculate number of NWP pressure levels
  nlev_std = n_elements(stdplev)

; determine size of NUCAPS arrays, i.e., number of retrievals
  nobs = n_elements(nucaps.latitude)
; define output array
  stdwv = fltarr(nlev_std,nobs)*!Values.F_NAN; mixing ratio [molec/cm2]
  stdoz = fltarr(nlev_std,nobs)*!Values.F_NAN; ozone mixing ratio [molec/cm2]
  stdco = fltarr(nlev_std,nobs)*!Values.F_NAN; ozone mixing ratio [molec/cm2]

; loop over each retrieval and interpolate profile by profile
  for iobs=0, nobs-1 do begin

   sfc = botlev(iobs)
   sfcp = nucaps.pressure(sfc,iobs)

; --------------------------
; START INTERPOLATION
; --------------------------
; This routine needs to output for trace gas values on standard Levels, not Layers 
; NUCAPS retrieves trace gases on layers, need to convert to levels
; Calculate retrieval column densities from pressure layers to pressure levels
; add two layers to make a level
; TOA layer = TOA level
;
;
; When you convert from pressure layers to levels, 
; you lose one level, the surface
; (1) define 100-level array to populate
    lev_wvcd = fltarr(nlev)*!Values.F_NAN; column density [molec/cm2]
    lev_ozcd = fltarr(nlev)*!Values.F_NAN; column density [molec/cm2]
    lev_cocd = fltarr(nlev)*!Values.F_NAN; 

; (2) Convert values at pressure LAYERS to values at pressure LEVELS
    lev_wvcd(0)=wvcd(0,iobs)
    lev_ozcd(0)=ozcd(0,iobs)
    lev_cocd(0)=cocd(0,iobs)

    for L=1,sfc+1 do begin
       lev_wvcd(L) = 0.5*wvcd(L-1,iobs) + 0.5*wvcd(L,iobs)
       lev_ozcd(L) = 0.5*ozcd(L-1,iobs) + 0.5*ozcd(L,iobs)
       lev_cocd(L) = 0.5*cocd(L-1,iobs) + 0.5*cocd(L,iobs)
    endfor
	
    idx=where(abs(stdplev-sfcp) eq min(abs(stdplev-sfcp)))
    stdp = stdplev(0:idx)

; You're not treating the boundary layer correctly here.  
; Trace gas interpolation must be done carefully to preserve mass
; NUCAPS retrieves the number of molecules and you don't want to lose any of them
; in the interpolation process.
;
; (3) Linearize LEVEL column density by interpolating the log form
    alog_wv = alog10(lev_wvcd) ; H2O column density
    alog_oz = alog10(lev_ozcd) ; ozone column density
    alog_co = alog10(lev_cocd) ; ozone column density
    alog_retp = alog10(pres) ; NUCAPS retrieval pressure grid
    alog_stdp = alog10(stdp) ; standard interpolation pressure grid

; (4) Interpolate log form 
    logwvcd = interpol(alog_wv,alog_retp,alog_stdp,/NaN)
    logozcd = interpol(alog_oz,alog_retp,alog_stdp,/NaN)
    logcocd = interpol(alog_co,alog_retp,alog_stdp,/NaN)

; (5) Restore column density to physical values from log form
    stdwv(0:idx,iobs)=10.0^(logwvcd); TOA to level column density on standard pressure levels
    stdoz(0:idx,iobs)=10.0^(logozcd);
    stdco(0:idx,iobs)=10.0^(logcocd);
; ---------------------------
; END INTERPOLATION
; ---------------------------
  endfor; end loop over iobs
    print,'Trace gas interpolation done'
end

