pro nsprep_co_conv,retfn,akfn,co_apr,co_ret,finv,finex

; =======================================================
;    START HEADER
; =======================================================
; INPUT: 
; - retfn: netCDF NUCAPS EDR file
; 	e.g., '/ships19/aqda/nadias/2019/07/22/retfiles/NUCAPS-sciEDR_am_npp_s20190722010807_e20190722011605_STC_fsr.nc'
; - akfn: netCDF NUCAPS averaging kernel file
; 	e.g., '/ships19/aqda/nadias/2019/07/22/akfiles/nucaps_n20_am_s201907220108070_e201907220116050_AK.nc'

; OUTPUT:
; - co_fg [nlev,nobs] NUCAPS CO first guess [molec/cm2]
; - co_ret [nlev,nobs] NUCAPS CO retrieval [molec/cm2]
; - finv, 
; - finex 

; Dependencies: 
; > /ships19/aqda/nadias/code/postprocessor/idl/read_ncdf.pro
; > /ships19/aqda/nadias/code/postprocessor/convert_AK/calc_finv.pro
;
; ***** IMPORTANT NOTE ******
; This code has not been tested on data
; Sample code only to illustrate concepts
; ***************************
;
; Nadia Smith
; nadias@stcnet.com
; 22 April 2020
;
; ========================================================
;     END HEADER
; ========================================================

; (1) Read NUCAPS CO retrieval
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; ******** IMPORTANT NOTE ABOUT CO RETRIEVAL PROFILE *****
; No need to adjust boundary layer of retrieval profile
; /ships19/aqda/nadias/code/postprocessor/convertEDR_ncdf/convertEDR.F already adjusts the boundary layer
;
; ALL values below the boundary layer are set to -9999.
; The last value in the profile -ne -9999. is the value in the boundary layer corrected for surface pressure
; Use this profile as is. 
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; according to surface pressure. 
out=read_ncdf(retfn,l2_ret)
co_ret=l2_ret.CO; [nlev x nobs], where nlev=100, nobs=1800 
; x_r is already adjusted for surface pressure. NO NEED TO MULTIPLY WITH PMULT
s=size(co_ret)
ret_nlev=s(1)
nobs=s(2)
pmult=l2_ret.pmult; [nlev x nobs]
; need pmult to multiply with x_a to adjust for surface pressure

; (2) Read NUCAPS AK and CO a-priori
out=read_ncdf(akfn,l2_ak)

; l2_ak.co_fg [nobs x nlev]
; l2_ak.co_func_pres [9]: CO trapezoid pressure layers
; l2_ak.co_func_last_indx [nobs]: trapezoid layer intersecting surface pressure
; l2_ak.co_ave_kern: [nobs x 9 x 9]: CO AK matrix on trapezoid layers
; l2_ak.co_bot_func_pres_eff [nobs]; pressure of trapezoid layer intersection surf pres

; (3) Assign full reconstructed AK matrix as reported in files. 
; ---------------------------------------
; RECONSTRUCT THE AK MATRIX (ndim x ndim)
; ---------------------------------------
; - ak_conv.pro: ak_rec, ndim, lambda_c, phi_lim, lambda_vec, eigvec_mat, dof, ak
; - NO NEED TO RUN AK_CONV
; - read reconstructed ak from the AK file
ak = l2_ak.co_ave_kern
ak_nlev = n_elements(l2_ak.co_func_indxs); 10 for CO

; (4) Adjust trapezoid pressure layers to surface pressure
; ak_conv.pro: changlev, f93.Psurf, pobs, numlev, nlev, lev_in, nlev_fin, lev_fin
; 	nlev = n_elements(l2_ak.co_func_indxs)
; 	nlev_fin = l2_ak.co_func_last_indx
; 	lev_in = l2_ak.co_func_indxs
;       lev_fin = lev_in(0:nlev_fin-1)
; - NO NEED TO RUN CHANGLEV – CAN DERIVE ALL FROM NETCDF FIELDS
; - this section of code replaces the changelev.pro routine
lev_in = l2_ak.co_func_indxs;[ak_nlev]
lev_fin = intarr(ak_nlev,nobs)-9999; number of trapezoid layers above surface at retrieval site
nlev_fin=intarr(nobs)-9999

for iobs = 0,nobs-1 do begin
	lsurf = l2_ak.co_func_last_indx(iobs); 
	lev_fin(0:lsurf-1,iobs)=lev_in(0:lsurf-1)
	nlev_fin(iobs)=lsurf
endfor

; (5) Adjust CO first guess to surface pressure
co_apr=fltarr(ret_nlev,nobs)-9999.
for iobs=0,nobs-1 do begin
	idx=where(pmult(*,iobs) gt 0)
	co_apr(idx,iobs)=l2_ak.co_fg(iobs,idx)*pmult(idx,iobs)
endfor

; (6) Read 'truth' file
; <insert routine> with output 'x_t'

;---------------
; CONVOLVE TRUTH
;---------------
; Degrade the truth onto the trapezoid space, convolve it to the ave,
; bring it back to 100 level grid. this is the convoluted truth to
; compare to the retrieval

; Indicate the type of interpolation to compute pseudo inverse of
; trapezoid matrix
style = 1L
htop = 1
hbot = 1

; Retreival pressure layers
; loop over all retrieval scenes
for iobs = 0,nobs-1 do begin
	x_a = co_apr(*,iobs); a priori 
	x_r = co_ret(*,iobs); retrieval
; Now compute pseudo inverse of trapezois matrix
; Equation 11 of Maddy and Barnet, IEEE TGRS, 2008 (MB08) 
;   calc_finv, style, nlev_fin, lev_fin, numlev, htp, hbt, pobs, finv, $
;           fine=finex
      ilev = where(pmult(*,iobs) gt 0.0); indices of pressure layers above surface
      ret_nlev = n_elements(ilev); number pressure layers above surface at retrieval site  
      ret_plev = l2_ret.effective_pressure(ilev,iobs); pressure [hPa] above surface
      trap_nlev = nlev_fin(iobs); number of trapezoide pressure hingepoints
      trap_plev_idx = lev_fin(0:trap_nlev-1,iobs); indices of trapezoid pressure hingepoints
      calc_finv, style, trap_nlev, trap_plev_idx, ret_nlev, htop, hbot, ret_plev, finv, $
                        fine=finex

; project gas profile onto trapezoid basis using the
; pseudo inverse of trapezoid matrix Eq. 13 of MB08
;                delta_x = ALOG(x_t[0:for_nlev-1]) - ALOG(x_a[0:for_nlev-1])

; convolve profile to ak and project back to fine grid 
; Equation 13, 16-17 of Maddy and Barnet, IEEE TGRS, 2008 (MB08)
; ln(cd_t_conv(l)) = ln(cd_a(l)) + F(l,j)*AK(j,j)#Finv(j,l)#ln(cd_t(l) / cd_a(l))
;                x_t_conv = EXP( ALOG(x_a) + TRANSPOSE(finex)#ak#finv#delta_x)
;                truth_colv(0:for_nlev-1,i) = x_t_conv
endfor; end loop over nfoot

end
